@@ -85,7 +85,7 @@ MIfitHMM <- function(miData, ...) {
 #' @param prior A function that returns the log-density of the working scale parameter prior distribution(s).  See \code{\link{fitHMM}}.
 #' @param modelName An optional character string providing a name for the fitted model. If provided, \code{modelName} will be returned in \code{\link{print.momentuHMM}}, \code{\link{AIC.momentuHMM}}, \code{\link{AICweights}}, and other functions. 
 #' @param covNames Names of any covariates in \code{miData$crwPredict} (if \code{miData} is a \code{\link{crwData}} object; otherwise 
-#' \code{covNames} is ignored). See \code{\link{prepData}}. 
+#' \code{covNames} is ignored). See \code{\link{prepData}}. If \code{miData} is a \code{\link{crwData}} object, any covariate in \code{miData$crwPredict} that is used in \code{formula}, \code{formulaDelta}, \code{formulaPi}, or \code{DM} must be included in \code{covNames}.
 #' @param spatialCovs List of raster layer(s) for any spatial covariates. See \code{\link{prepData}}. 
 #' @param centers 2-column matrix providing the x-coordinates (column 1) and y-coordinates (column 2) for any activity centers (e.g., potential 
 #' centers of attraction or repulsion) from which distance and angle covariates will be calculated based on realizations of the position process. 
@@ -188,9 +188,6 @@ MIfitHMM <- function(miData, ...) {
 #' 
 #' @export
 #' @importFrom crawl crwPostIS crwSimulator
-#' @importFrom doParallel registerDoParallel stopImplicitCluster
-#' @importFrom foreach foreach %dopar%
-#' @importFrom doRNG %dorng%
 # @importFrom raster getZ
 #' @importFrom stats terms.formula
 MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha = 0.95, na.rm = FALSE,
@@ -235,6 +232,8 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
       stop("Either nbStates and dist must be specified (for a regular HMM) or hierStates and hierDist must be specified (for a hierarchical HMM)")
   }
   
+  chkDots(...)
+  
   j <- mf <- mD <- NULL #gets rid of no visible binding for global variable NOTE in R cmd check
   
   if(poolEstimates){
@@ -243,6 +242,28 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
     } else if(optMethod %in% fitMethods[-1] & !is.null(control$hessian)){
       if(!control$hessian) stop("estimates cannot be pooled unless hessian is calculated")
     }
+    if (!requireNamespace("mitools", quietly = TRUE)) {
+      stop("Package \"mitools\" needed for this function to work. Please install it.",
+           call. = FALSE)
+    }
+  }
+  
+  if(ncores>1){
+    for(pkg in c("doFuture","future")){
+      if (!requireNamespace(pkg, quietly = TRUE)) {
+        stop("Package \"",pkg,"\" needed for parallel processing to work. Please install it.",
+             call. = FALSE)
+      }
+    }
+    oldDoPar <- doFuture::registerDoFuture()
+    on.exit(with(oldDoPar, foreach::setDoPar(fun=fun, data=data, info=info)), add = TRUE)
+    future::plan(future::multisession, workers = ncores)
+    # hack so that foreach %dorng% can find internal momentuHMM variables without using ::: (forbidden by CRAN)
+    progBar <- progBar
+    pkgs <- c("momentuHMM")
+  } else { 
+    doParallel::registerDoParallel(cores=ncores)
+    pkgs <- c("momentuHMM")
   }
   
   if(is.crwData(miData)){
@@ -283,25 +304,27 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
       } else {
         distnames <- tmpdistnames <- names(predData)[which(!(names(predData) %in% c("ID",Time.name,c("mu.x","mu.y"),covNames,znames)))]
       }
-      cat('Drawing ',nSims,' realizations from the position process using crawl... ',ifelse(ncores>1 & length(ids)>1,"","\n"),sep="")
-      
+      cat('Drawing ',nSims,' realizations from the position process using crawl... \n',sep="")
+      if(ncores>1) message("Running simulator in parallel... ")
       nbAnimals <- length(ids)
       predTimes <- lapply(ids,function(x) predData[[Time.name]][which(predData$ID==x & predData$locType=="p")])
       names(predTimes) <- ids
-      registerDoParallel(cores=ncores)
-      withCallingHandlers(crwSim <- foreach(mf = model_fits, i = ids, .export="crwSimulator") %dorng% {
-          cat("Simulating individual ",i,"...\n",sep="")
-          crawl::crwSimulator(mf,predTime=predTimes[[i]], method = method, parIS = parIS,
-                                  df = dfSim, grid.eps = grid.eps, crit = crit, scale = scaleSim, quad.ask = ifelse(ncores>1, FALSE, quad.ask), force.quad = force.quad)
+      withCallingHandlers(crwSim <- foreach(mf = model_fits, i = ids, .export="crwSimulator", .packages=pkgs) %dorng% {
+          if(ncores==1) message("Running simulator for individual ",i,"... ",sep="")
+          else progBar(which(ids==i), length(ids))
+          crSim <- suppressMessages(crawl::crwSimulator(mf,predTime=predTimes[[i]], method = method, parIS = parIS,
+                                  df = dfSim, grid.eps = grid.eps, crit = crit, scale = scaleSim, quad.ask = ifelse(ncores>1, FALSE, quad.ask), force.quad = force.quad))
+          return(crSim)
       },warning=muffleRNGwarning)
-      stopImplicitCluster()
-      names(crwSim) <- ids
       if(ncores==1) cat("DONE\n")
+      names(crwSim) <- ids
       
-      registerDoParallel(cores=ncores)
+      
+      if(ncores>1) message("Drawing imputations in parallel... ",sep="")
       withCallingHandlers(miData<-
-        foreach(j = 1:nSims, .export=c("crwPostIS","prepData"), .errorhandling="pass") %dorng% {
-          cat("\rDrawing imputation ",j,"... ",sep="")
+        foreach(j = 1:nSims, .export=c("crwPostIS","prepData"), .errorhandling="pass", .packages=pkgs) %dorng% {
+          if(ncores==1) message("\rDrawing imputation ",j,"... ",sep="")
+          else progBar(j, nSims)
           locs<-data.frame()
           for(i in 1:length(ids)){
               tmp<-tryCatch({crawl::crwPostIS(crwSim[[i]], fullPost = fullPost, df = dfPostIS, scale = scalePostIS, thetaSamp = thetaSamp)},error=function(e) e)
@@ -309,18 +332,22 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
               locs<-rbind(locs,tmp$alpha.sim[,c("mu.x","mu.y")])
           }
           df<-data.frame(x=locs$mu.x,y=locs$mu.y,predData[,c("ID",Time.name,tmpdistnames,covNames,znames),drop=FALSE])[which(predData$locType=="p"),]
-          pD <- tryCatch(prepData.default(df,covNames=covNames,spatialCovs=spatialCovs,centers=centers,centroids=centroids,angleCovs=angleCovs,altCoordNames=altCoordNames),error=function(e) e)
-          pD
+          pD <- tryCatch(prepData(df,covNames=covNames,spatialCovs=spatialCovs,centers=centers,centroids=centroids,angleCovs=angleCovs,altCoordNames=altCoordNames),error=function(e) e)
+          return(pD)
         }
       ,warning=muffleRNGwarning)
-      stopImplicitCluster()
-      cat("DONE\n")
+      if(ncores==1) cat("DONE\n")
+      
       for(i in which(unlist(lapply(miData,function(x) inherits(x,"error"))))){
         warning('prepData failed for imputation ',i,"; ",miData[[i]])
       }
       ind <- which(unlist(lapply(miData,function(x) inherits(x,"momentuHMMData"))))
       if(fit) cat('Fitting',length(ind),'realizations of the position process using fitHMM... \n')
-      else return(crwSim(list(miData=miData,crwSimulator=crwSim)))
+      else {
+        if(ncores==1) doParallel::stopImplicitCluster()
+        else future::plan(future::sequential)
+        return(crwSim(list(miData=miData,crwSimulator=crwSim)))
+      }
     } else stop("nSims must be >0")
     
   } else {
@@ -390,6 +417,7 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
     parallelStart <- 2
 
     cat("\rImputation ",1,"... ",sep="")
+    if(retryFits>=1) cat("\n")
     
     fits[[1]]<-suppressMessages(fitHMM.momentuHMMData(miData[[1]],nbStates, dist, Par0[[1]], beta0[[1]], delta0[[1]],
                                     estAngleMean, circularAngleMean, formula, formulaDelta, stationary, mixtures, formulaPi,
@@ -398,7 +426,7 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
     if(retryFits>=1){
       cat("\n")
     }
-    cat("DONE\nFitting remaining imputations... \n")
+    cat("DONE\n")
 
     tmpPar <- getPar0(fits[[1]])
     Par0[parallelStart:nSims] <- list(tmpPar$Par)
@@ -412,14 +440,18 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
     if(!is.null(control)) control$trace <- 0
   }
   
-  registerDoParallel(cores=ncores)
+  if(useInitial | ncores>1) cat("Fitting ",ifelse(useInitial,"remaining ",""),length(parallelStart:nSims)," imputation",ifelse(length(parallelStart:nSims)>1,"s",""),ifelse(ncores>1," in parallel",""),"... \n",sep="")
+  
   withCallingHandlers(fits[parallelStart:nSims] <-
-    foreach(mD = miData[parallelStart:nSims], j = parallelStart:nSims, .export=c("fitHMM.momentuHMMData"), .errorhandling="pass") %dorng% {
+    foreach(mD = miData[parallelStart:nSims], j = parallelStart:nSims, .export=c("fitHMM"), .errorhandling="pass", .packages=pkgs) %dorng% {
       
-      if(nSims>1) {
+      if(nSims==1 | ncores==1 | retryFits>=1) {
         cat("     \rImputation ",j,"... ",sep="")
+        if(retryFits>=1) cat("\n")
+      } else {
+        if(retryFits<1) progBar(j,nSims)
       }
-      tmpFit<-suppressMessages(fitHMM.momentuHMMData(mD,nbStates, dist, Par0[[j]], beta0[[j]], delta0[[j]],
+      tmpFit<-suppressMessages(fitHMM(mD,nbStates, dist, Par0[[j]], beta0[[j]], delta0[[j]],
                                       estAngleMean, circularAngleMean, formula, formulaDelta, stationary, mixtures, formulaPi,
                                       nlmPar, fit, DM,
                                       userBounds, workBounds, betaCons, betaRef, deltaCons, mvnCoords, stateNames, knownStates[[j]], fixPar, retryFits, retrySD, optMethod, control, prior, modelName))
@@ -427,8 +459,10 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
       tmpFit
     } 
   ,warning=muffleRNGwarning)
-  stopImplicitCluster()
-  cat("DONE\n")
+  if((nSims>1 & ncores==1) | retryFits>=1) cat("DONE\n")
+  
+  if(ncores==1) doParallel::stopImplicitCluster()
+  else future::plan(future::sequential)
   
   for(i in which(!unlist(lapply(fits,function(x) inherits(x,"momentuHMM"))))){
     warning('Fit #',i,' failed; ',fits[[i]])
@@ -456,11 +490,8 @@ MIfitHMM.default<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha
 #' 
 #' @export
 #' @importFrom crawl crwPostIS crwSimulator
-#' @importFrom doParallel registerDoParallel stopImplicitCluster
-#' @importFrom foreach foreach %dopar%
-#' @importFrom doRNG %dorng%
 # @importFrom raster getZ
-#' @importFrom data.tree Clone
+# #' @importFrom data.tree Clone
 MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE, alpha = 0.95, na.rm = FALSE,
                        hierStates, hierDist, 
                        Par0, hierBeta = NULL, hierDelta = NULL,
@@ -482,6 +513,32 @@ MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE,
     } else if(optMethod %in% fitMethods[-1] & !is.null(control$hessian)){
       if(!control$hessian) stop("estimates cannot be pooled unless hessian is calculated")
     }
+    if (!requireNamespace("mitools", quietly = TRUE)) {
+      stop("Package \"mitools\" needed for this function to work. Please install it.",
+           call. = FALSE)
+    }
+  }
+  
+  chkDots(...)
+  
+  installDataTree()
+  
+  if(ncores>1){
+    for(pkg in c("doFuture","future")){
+      if (!requireNamespace(pkg, quietly = TRUE)) {
+        stop("Package \"",pkg,"\" needed for parallel processing to work. Please install it.",
+             call. = FALSE)
+      }
+    }
+    oldDoPar <- doFuture::registerDoFuture()
+    on.exit(with(oldDoPar, foreach::setDoPar(fun=fun, data=data, info=info)), add = TRUE)
+    future::plan(future::multisession, workers = ncores)
+    # hack so that foreach %dorng% can find internal momentuHMM variables without using ::: (forbidden by CRAN)
+    progBar <- progBar
+    pkgs <- c("momentuHMM","data.tree")
+  } else { 
+    doParallel::registerDoParallel(cores=ncores)
+    pkgs <- NULL
   }
   
   if(is.crwHierData(miData)){
@@ -523,25 +580,26 @@ MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE,
       } else {
         distnames <- tmpdistnames <- names(predData)[which(!(names(predData) %in% c("ID",Time.name,"level","locType",c("mu.x","mu.y"),covNames,znames)))]
       }
-      cat('Drawing ',nSims,' realizations from the position process using crawl... ',ifelse(ncores>1 & length(ids)>1,"","\n"),sep="")
-      
+      cat('Drawing ',nSims,' realizations from the position process using crawl... \n',sep="")
+      if(ncores>1) message("Running simulator in parallel... ")
       nbAnimals <- length(ids)
       predTimes <- lapply(ids,function(x) predData[[Time.name]][which(predData$ID==x & predData$locType=="p")])
       names(predTimes) <- ids
-      registerDoParallel(cores=ncores)
-      withCallingHandlers(crwHierSim <- foreach(mf=model_fits, i=ids, .export="crwSimulator") %dorng% {
-        cat("Simulating individual ",i,"...\n",sep="")
-        crawl::crwSimulator(mf,predTime=predTimes[[i]], method = method, parIS = parIS,
-                            df = dfSim, grid.eps = grid.eps, crit = crit, scale = scaleSim, quad.ask = ifelse(ncores>1, FALSE, quad.ask), force.quad = force.quad)
+      withCallingHandlers(crwHierSim <- foreach(mf=model_fits, i=ids, .export="crwSimulator", .packages = pkgs) %dorng% {
+        if(ncores==1) message("Running simulator for individual ",i,"... ",sep="")
+        else progBar(which(ids==i), length(ids))
+        crSim <- suppressMessages(crawl::crwSimulator(mf,predTime=predTimes[[i]], method = method, parIS = parIS,
+                            df = dfSim, grid.eps = grid.eps, crit = crit, scale = scaleSim, quad.ask = ifelse(ncores>1, FALSE, quad.ask), force.quad = force.quad))
+        return(crSim)
       },warning=muffleRNGwarning)
-      stopImplicitCluster()
-      names(crwHierSim) <- ids
       if(ncores==1) cat("DONE\n")
+      names(crwHierSim) <- ids
       
-      registerDoParallel(cores=ncores)
+      if(ncores>1) message("Drawing imputations in parallel... ",sep="")
       withCallingHandlers(miData<-
-                            foreach(j = 1:nSims, .export=c("crwPostIS","prepData"), .errorhandling="pass") %dorng% {
-                              cat("\rDrawing imputation ",j,"... ",sep="")
+                            foreach(j = 1:nSims, .export=c("crwPostIS","prepData"), .errorhandling="pass", .packages=pkgs) %dorng% {
+                              if(ncores==1) message("\rDrawing imputation ",j,"... ",sep="")
+                              else progBar(j, nSims)
                               locs<-data.frame()
                               for(i in 1:length(ids)){
                                 #if(!is.null(model_fits[[i]]$err.model)){
@@ -558,18 +616,22 @@ MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE,
                               df[which(df$locType=="p"),"x"] <- locs[which(locs$locType=="p"),"x"]
                               df[which(df$locType=="p"),"y"] <- locs[which(locs$locType=="p"),"y"]
                               df$locType <- NULL
-                              pD <- tryCatch(prepData.hierarchical(df,covNames=covNames,spatialCovs=spatialCovs,centers=centers,centroids=centroids,angleCovs=angleCovs,altCoordNames=altCoordNames,hierLevels=levels(predData$level),coordLevel=attr(predData,"coordLevel")),error=function(e) e)
-                              pD
+                              pD <- tryCatch(prepData(df,covNames=covNames,spatialCovs=spatialCovs,centers=centers,centroids=centroids,angleCovs=angleCovs,altCoordNames=altCoordNames,hierLevels=levels(predData$level),coordLevel=attr(predData,"coordLevel")),error=function(e) e)
+                              return(pD)
                             }
                           ,warning=muffleRNGwarning)
-      stopImplicitCluster()
-      cat("DONE\n")
+      if(ncores==1) cat("DONE\n")
+      
       for(i in which(unlist(lapply(miData,function(x) inherits(x,"error"))))){
         warning('prepData failed for imputation ',i,"; ",miData[[i]])
       }
       ind <- which(unlist(lapply(miData,function(x) inherits(x,"momentuHierHMMData"))))
       if(fit) cat('Fitting',length(ind),'realizations of the position process using fitHMM... \n')
-      else return(crwHierSim(list(miData=miData,crwSimulator=crwHierSim)))
+      else {
+        if(ncores==1) doParallel::stopImplicitCluster()
+        else future::plan(future::sequential)
+        return(crwHierSim(list(miData=miData,crwSimulator=crwHierSim)))
+      }
     } else stop("nSims must be >0")
     
   } else {
@@ -606,7 +668,7 @@ MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE,
     if(!is.list(hierBeta)){
       if(!inherits(hierBeta,"Node"))
         tmphierBeta<-hierBeta
-      else tmphierBeta <- Clone(hierBeta)
+      else tmphierBeta <- data.tree::Clone(hierBeta)
       hierBeta<-vector('list',nSims)
       if(!is.null(tmphierBeta))
         hierBeta[1:nSims]<-list(tmphierBeta)
@@ -680,25 +742,29 @@ MIfitHMM.hierarchical<-function(miData,nSims, ncores = 1, poolEstimates = TRUE,
   if(nSims>1 & ncores>1){
     if(!is.null(nlmPar)) nlmPar$print.level <- 0
     if(!is.null(control)) control$trace <- 0
+    message("        Fitting ",length(parallelStart:nSims)," imputation",ifelse(length(parallelStart:nSims)>1,"s","")," in parallel... ",sep="")
   }
   
-  registerDoParallel(cores=ncores)
   withCallingHandlers(fits[parallelStart:nSims] <-
-                        foreach(mD = miData[parallelStart:nSims], j = parallelStart:nSims, .export=c("fitHMM.momentuHierHMMData"), .errorhandling="pass") %dorng% {
+                        foreach(mD = miData[parallelStart:nSims], j = parallelStart:nSims, .export=c("fitHMM"), .errorhandling="pass", .packages = pkgs) %dorng% {
                           
-                          if(nSims>1) {
+                          if(nSims==1 & ncores==1) {
                             cat("     \rImputation ",j,"... ",sep="")
+                          } else {
+                            progBar(j,nSims)
                           }
-                          tmpFit<-suppressMessages(fitHMM.momentuHierHMMData(mD, hierStates, hierDist, Par0[[j]], hierBeta[[j]], hierDelta[[j]],
+                          tmpFit<-suppressMessages(fitHMM(mD, hierStates, hierDist, Par0[[j]], hierBeta[[j]], hierDelta[[j]],
                                                               estAngleMean, circularAngleMean, hierFormula, hierFormulaDelta, mixtures, formulaPi, 
                                                               nlmPar, fit, DM, 
                                                               userBounds, workBounds, betaCons, deltaCons, mvnCoords, knownStates[[j]], fixPar, retryFits, retrySD, optMethod, control, prior, modelName))
-                          if(retryFits>=1) cat("\n")
+                          if(retryFits>=1 & ncores==1) cat("\n")
                           tmpFit
                         } 
                       ,warning=muffleRNGwarning)
-  stopImplicitCluster()
-  cat("DONE\n")
+  if(nSims>1 & ncores==1) cat("DONE\n")
+  
+  if(ncores==1) doParallel::stopImplicitCluster()
+  else future::plan(future::sequential)
   
   for(i in which(!unlist(lapply(fits,function(x) inherits(x,"momentuHierHMM"))))){
     warning('Fit #',i,' failed; ',fits[[i]])
